#include <stdlib.h>
#include <stdio.h>
#include <sys/time.h>
#include <omp.h>
#include <signal.h>
#include <unistd.h>
//#include <iostream>
//#include <iomanip>
//#include <omp.h>
//#include <mpi.h>

//using namespace std;

// Scheme for the problem of Successive squares of submatrices - CUDA version
// Spanish Parallel Programming Contest 2015, problem B

void initialize(int t,double *a,int lv,int uv)
{
  int i;

  for(i=0;i<t*t;i++)
  {
//the double elements are generated with three decimals, between lv and uv.
     a[i]=((int) ((((1.*rand())/RAND_MAX)*(uv-lv)+lv)*1000.))/1000.;
  }
}

void escribir(int t,double *a){
  int i, j;

  for (i = 0; i < t; i++)
  {
    for (j = 0; j < t; j++)
    {  
      printf("%.4lf ",a[i*t+j]);
    }
    printf("\n");
  }
  printf("\n");
}

void escribirresult(double *a,int t,int step)
{
  int i,j;

  for(i=0;i<t;i++)
  {
     for(j=0;j<t;j++)
     {
        if(((i*t+j)%step)==0)
        {
          printf("%.4lf ",a[i*t+j]);
        }
     }
     printf("\n");
  }
  printf("\n");
}

/*
c
c     mseconds - returns elapsed milliseconds since Jan 1st, 1970.
c
*/
long long mseconds(){
  struct timeval t;
  gettimeofday(&t, NULL);
  return t.tv_sec*1000 + t.tv_usec/1000;
}

static void alarm_handler(int sig) {
  fprintf(stderr, "Time Limit Exceeded\n");
  abort();
}

extern void sec(int,int,int,int,double *);

int main(int argc,char *argv[]) {
//  int nodo,np; //process identifier and total number of processes
  int num_problems; //number of problems  
  int N; //size of matrice
  int M; //size of submatrices
  int L; //size of overlap
  int S; //number of steps
  int seed; //seed for random generation  
  int lower_value; //lower value for the random generation
  int upper_value; //upper value for the random generation
  int gap; //gap between values of the matrices to write in the output
  long long ti,tf,tt=0; //initial, final and total times
  double *A; //matrix of data
  int i,j; //auxiliar indices

  FILE *stats_file = fopen("stats", "w");

//to stablish a time limit
  struct sigaction sact;
  sigemptyset(&sact.sa_mask);
  sact.sa_flags = 0;
  sact.sa_handler = alarm_handler;
  sigaction(SIGALRM, &sact, NULL);
  alarm(60);  /* time limit */
 
//  MPI_Init(&argc,&argv);
//  MPI_Comm_size(MPI_COMM_WORLD,&np);
//  MPI_Comm_rank(MPI_COMM_WORLD,&nodo);

// The number of test cases is read
// and sent to all the processes
//  if(nodo==0)
//  {
    scanf("%d",&num_problems);
//    MPI_Bcast(&num_problems,1,MPI_INT,0,MPI_COMM_WORLD);
//  }
//  else
//  {
//    MPI_Bcast(&num_problems,1,MPI_INT,0,MPI_COMM_WORLD);
//  }
  for(i=0;i<num_problems;i++)
  {
 //   if(nodo==0)
 //   {
      scanf("%d",&N);                   // The first argument is the size of the matrix
      scanf("%d",&M);             // the second is the size of submatrices
      scanf("%d",&L);	//the third is the number of rows-columns overlaping
      scanf("%d",&S);  //the fourth is the number of steps
      scanf("%d",&seed); //seed for the random generation of data
      scanf("%d",&lower_value);  //lower bound for the values in the matrix
      scanf("%d",&upper_value);  //upper bound for the values in the matrix
      scanf("%d",&gap); //gap between values of the matrices to write in the output
// Space for the matrix
      A = (double *) malloc(sizeof(double)*N*N);

      srand(seed);
      initialize(N,A,lower_value,upper_value);

#ifdef DEBUG
// While debugging the matrix written
      escribir(N,A);
#endif

//    }
//    MPI_Barrier(MPI_COMM_WORLD);
    ti=mseconds(); 
    sec(N,M,L,S,A);
//    MPI_Barrier(MPI_COMM_WORLD);
    tf=mseconds(); 
//    if(nodo==0)
//    {
// The time of the first input is not considered
      if(i!=0) tt+=tf-ti;
#ifdef DEBUG
// While debugging the time of each execution is written
      fprintf(stats_file, "%Ld\n", tf-ti);
#endif
// The results of each problem are written,
// even for the first problem
      escribirresult(A,N,gap);
      free(A);
//    } 
  }
  
//  if(nodo==0)
//  { 
    fprintf(stats_file, "%Ld\n", tt);
    fclose(stats_file);
//  }
//  MPI_Finalize();
  return 0;
}
