#include <stdlib.h>
#include <stdio.h>
#include <sys/time.h>
#include <omp.h>
#include <signal.h>
#include <unistd.h>
//#include <iostream>
//#include <iomanip>
//#include <omp.h>

//using namespace std;

// Scheme for the problem of Sum of rectangular submatrices - OpenMP version
// Spanish Parallel Programming Contest 2015, problem C

void initializedouble(int n,int m,double *a,int lv,int uv)
{
  int i;

  for(i=0;i<n*m;i++)
  {
//the double elements are generated with three decimals, between lv and uv
     a[i]=((int) ((((1.*rand())/RAND_MAX)*(uv-lv)+lv)*1000.))/1000.;
  }
}

void initializenatural(int n,int m,int *a,int lv,int uv)
{
  int i;

  for(i=0;i<n*m;i++)
  {
      a[i]=(int) (((1.*rand())/RAND_MAX)*(uv-lv)+lv);
  }
}

void escribirdouble(int n,int m,double *a){
  int i, j;

  for (i = 0; i < n; i++)
  {
    for (j = 0; j < m; j++)
    {  
      printf("%.4lf ",a[i*m+j]);
    }
    printf("\n");
  }
  printf("\n");
}

void escribirnatural(int n,int m,int *a){
  int i, j;

  for (i = 0; i < n; i++)
  {
    for (j = 0; j < m; j++)
    {  
      printf("%d ",a[i*m+j]);
    }
    printf("\n");
  }
  printf("\n");
}

void escribirresult(int n,int m,double *a,int step)
{
  int i,j;

  for(i=0;i<n;i++)
  {
     for(j=0;j<m;j++)
     {
        if(((i*m+j)%step)==0)
        {
          printf("%.4lf ",a[i*m+j]);
        }
     }
     printf("\n");
  }
  printf("\n");
}

/*
c
c     mseconds - returns elapsed milliseconds since Jan 1st, 1970.
c
*/
long long mseconds(){
  struct timeval t;
  gettimeofday(&t, NULL);
  return t.tv_sec*1000 + t.tv_usec/1000;
}

static void alarm_handler(int sig) {
  fprintf(stderr, "Time Limit Exceeded\n");
  abort();
}

extern void sec(int,int,int *,int *,double *);

int main(int argc,char *argv[]) {
  int num_problems; //number of problems  
  int N; //number of rows of matrices
  int M; //number of columns of matrices
  int seed; //seed for random generation  
  int gap; //number of data between each output
  int lower_value;  //lower bound for the values in the matrix
  int upper_value;  //upper bound for the values in the matrix
  double *A; //matrix of data
  int *R; //matrix of rows
  int *C; //matrix of columns 
  long long ti,tf,tt=0; //initial, final and total times

  FILE *stats_file = fopen("stats", "w");

//to stablish a time limit
  struct sigaction sact;
  sigemptyset(&sact.sa_mask);
  sact.sa_flags = 0;
  sact.sa_handler = alarm_handler;
  sigaction(SIGALRM, &sact, NULL);
  alarm(60);  /* time limit */
 
//  MPI_Init(&argc,&argv);
//  MPI_Comm_size(MPI_COMM_WORLD,&np);
//  MPI_Comm_rank(MPI_COMM_WORLD,&nodo);

// The number of test cases is read
// and sent to all the processes
//  if(nodo==0)
//  {
    scanf("%d",&num_problems);
//    MPI_Bcast(&cuantos,1,MPI_INT,0,MPI_COMM_WORLD);
//  }
//  else
//  {
//    MPI_Bcast(&cuantos,1,MPI_INT,0,MPI_COMM_WORLD);
//  }
  for(int i=0;i<num_problems;i++)
  {
//    if(nodo==0)
//    {
      scanf("%d",&N);                   // The first argument is the number of rows of the matrices
      scanf("%d",&M;             // the second is the number of columns of the matrices
      scanf("%d",&seed); //seed for the random generation of data
      scanf("%d",&lower_value);  //lower bound for the values in the matrix
      scanf("%d",&upper_value);  //upper bound for the values in the matrix
      scanf("%d",&gap); //gap between values of the matrices to write in the output
// Space for the data
      A = new double[N*M];
      R = new int[N*M];
      C = new int[N*M];

      srand(seed);
      initializedouble(N,M,A,lower_value,upper_value);
      initializenatural(N,M,R,0,N/2);
      initializenatural(N,M,C,0,M/2);
#ifdef DEBUG
// While debugging the matrices are written
      escribirdouble(N,M,A);
      escribirnatural(N,M,R);
      escribirnatural(N,M,C);
#endif

//    }
//    MPI_Barrier(MPI_COMM_WORLD);
    ti=mseconds(); 
    sec(N,M,R,C,A);
//    MPI_Barrier(MPI_COMM_WORLD);
    tf=mseconds(); 
//    if(nodo==0)
//    {
// The time of the first input is not considered
      if(i!=0) tt+=tf-ti;
#ifdef DEBUG
// While debugging the time of each execution is written
      fprintf(stats_file, "%Ld\n", tf-ti);
#endif
// The results of each problem are written,
// even for the first problem
      escribirresult(N,M,A,gap);
      delete[] A;
      delete[] R;
      delete[] C;
//    } 
  }
  
//  if(nodo==0)
//  { 
    fprintf(stats_file, "%Ld\n", tt);
    fclose(stats_file);
//  }
//  MPI_Finalize();
  return 0;
}
