#include <stdlib.h>
#include <stdio.h>
#include <sys/time.h>
#include <omp.h>
#include <signal.h>
#include <unistd.h>
#include <iostream>
#include <iomanip>
#include <omp.h>

using namespace std;

// Scheme for the problem of Distance between sets of points - OpenMP version
// Spanish Parallel Programming Contest 2015, problem E

void initialize(int n,int *a,int lv,int uv)
{
  int i;

  for(i=0;i<n;i++)
  {
      a[i]=(int) (((1.*rand())/RAND_MAX)*(uv-lv)+lv);
  }
}

void escribir(int S,int *P,int *D){
  int i, j;
  int acum=0;

  cout <<S <<endl;
  for (i = 0; i < S; i++)
  {
    cout << P[i] << ": ";
    for (j = acum; j < acum+2*P[i]; j+=2)
    {  
      cout << D[j]<<" "<<D[j+1]<< " ; ";
    }
    acum+=(2*P[i]);
    cout <<endl;
  }
  cout <<endl;
}

/*
c
c     mseconds - returns elapsed milliseconds since Jan 1st, 1970.
c
*/
long long mseconds(){
  struct timeval t;
  gettimeofday(&t, NULL);
  return t.tv_sec*1000 + t.tv_usec/1000;
}

static void alarm_handler(int sig) {
  fprintf(stderr, "Time Limit Exceeded\n");
  abort();
}

extern float sec(int,int *,int *);

int main(int argc,char *argv[]) {
  int num_problems; //number of problems  
  int S; //number of sets
  int seed; //seed for random generation  
  int lower_size;  //lower bound for the size of the sets
  int upper_size;  //upper bound for the size of the sets
  int lower_value;  //lower bound for the coordinates of the points
  int upper_value;  //upper bound for the coordinates of the points
  int *P; //vector of sizes of sets
  int *D; //vector with the coordinates of the points, of size 2*(P[0+P[1]+...+P[S-1])
  long long ti,tf,tt=0; //initial, final and total times
  float opt;

  FILE *stats_file = fopen("stats", "w");

//to stablish a time limit
  struct sigaction sact;
  sigemptyset(&sact.sa_mask);
  sact.sa_flags = 0;
  sact.sa_handler = alarm_handler;
  sigaction(SIGALRM, &sact, NULL);
  alarm(60);  /* time limit */
 
//  MPI_Init(&argc,&argv);
//  MPI_Comm_size(MPI_COMM_WORLD,&np);
//  MPI_Comm_rank(MPI_COMM_WORLD,&nodo);

// The number of test cases is read
// and sent to all the processes
//  if(nodo==0)
//  {
    cin >> num_problems;
//    MPI_Bcast(&cuantos,1,MPI_INT,0,MPI_COMM_WORLD);
//  }
//  else
//  {
//    MPI_Bcast(&cuantos,1,MPI_INT,0,MPI_COMM_WORLD);
//  }
  for(int i=0;i<num_problems;i++)
  {
//    if(nodo==0)
//    {
      cin >>S;                   // The first argument is the number of sets
      cin >>seed; //seed for the random generation of data
      cin >>lower_size;  //lower bound for the sizes
      cin >>upper_size;  //upper bound for the sizes
      cin >>lower_value;  //lower bound for the coordinates
      cin >>upper_value;  //upper bound for the coordinates
// Space for the data
      P = new int[S];

      srand(seed);
      initialize(S,P,lower_size,upper_size);
      int sizeD=0;
      for(int j=0;j<S;j++)
      {
        sizeD+=2*P[j];
      }
      D=new int[2*sizeD];
      initialize(sizeD,D,lower_value,upper_value);
#ifdef DEBUG
// While debugging the matrix ix written
      escribir(S,P,D);
#endif

//    }
//    MPI_Barrier(MPI_COMM_WORLD);
    ti=mseconds(); 
    opt=sec(S,P,D);
//    MPI_Barrier(MPI_COMM_WORLD);
    tf=mseconds(); 
//    if(nodo==0)
//    {
// The time of the first input is not considered
      if(i!=0) tt+=tf-ti;
#ifdef DEBUG
// While debugging the time of each execution is written
      fprintf(stats_file, "%Ld\n", tf-ti);
#endif
// The results of each problem are written,
// even for the first problem
      cout << setiosflags(ios::fixed) << setprecision(4) << opt<<endl;
     delete[] D;
     delete[] P; 
//    } 
  }
  
//  if(nodo==0)
//  { 
    fprintf(stats_file, "%Ld\n", tt);
    fclose(stats_file);
//  }
//  MPI_Finalize();
  return 0;
}
